@@ -47,6 +47,7 @@ struct _GnomePilotDruidPrivate
 	gint handle;
 
 	gboolean finished;
+	gboolean started;
 	
 	PilotState *state;
 	PilotState *orig_state;
@@ -65,11 +66,18 @@ struct _GnomePilotDruidPrivate
 
 	GtkWidget *device_name;
 	GtkWidget *device_port;
+	GtkWidget *device_port_combo;
+	GtkWidget *device_port_label;
 	GtkWidget *device_speed;
+	GtkWidget *device_speed_label;
 	GtkWidget *device_timeout;
 	GtkWidget *device_usb;
 	GtkWidget *device_irda;
 	GtkWidget *device_network;
+#ifdef PILOT_LINK_0_12
+	GtkWidget *libusb_label;
+	GList *libusb_list;
+#endif
 	
 	GtkWidget *pilot_info;
 	GtkWidget *pilot_info_no;
@@ -90,12 +98,17 @@ static gboolean get_widgets (GnomePilotDruid *gpd);
 static void map_widgets (GnomePilotDruid *gpd);
 static void init_widgets (GnomePilotDruid *gpd);
 static void fill_widgets (GnomePilotDruid *gpd);
+static void set_widget_visibility_by_type(GnomePilotDruid *gpd, int type);
+static void network_device_toggled_callback (GtkRadioButton *network_radio,
+    void *data);
 
 static gboolean gpd_delete_window (GtkWidget *widget,GdkEvent *event,gpointer user_data);
 static void gpd_canceled (GnomeDruid *druid, gpointer user_data);
 static void gpd_help (GnomeDruid *druid, gpointer user_data);
 
 static void gpd_cradle_page_prepare (GnomeDruidPage *page, gpointer arg1, gpointer user_data);
+static gboolean gpd_cradle_page_next (GnomeDruidPage *page, gpointer arg1, gpointer user_data);
+
 static void gpd_sync_page_prepare (GnomeDruidPage *page, gpointer arg1, gpointer user_data);
 static gboolean gpd_sync_page_back (GnomeDruidPage *druidpage, gpointer arg1, gpointer user_data);
 static gboolean gpd_pilot_page_two_next (GnomeDruidPage *druidpage, gpointer arg1, gpointer user_data);
@@ -157,6 +170,7 @@ init (GnomePilotDruid *gpd)
 	gpd->priv = priv;
 
 	priv->finished = FALSE;
+	priv->started = FALSE;
 	
 	/* State information */
 	loadPilotState (&priv->orig_state);
@@ -224,18 +238,31 @@ get_widgets (GnomePilotDruid *gpd)
 	priv->page_conduits = GW ("druidpage_conduits");
 	priv->page_finish = GW ("druidpage_finish");
 	
-	priv->device_name = GW ("device_name_entry");
-	priv->device_port = GW ("device_port_entry");
-	priv->device_speed = GW ("device_speed_menu");
-	priv->device_timeout = GW ("device_timeout_spinner");
-	priv->device_usb = GW ("usb_radio");
-	priv->device_irda = GW ("irda_radio");
-	priv->device_network = GW ("network_radio");
+	priv->device_name = GW ("druid_device_name_entry");
+	priv->device_port = GW ("druid_device_port_entry");
+	priv->device_port_label = GW ("druid_device_port_label");
+	priv->device_port_combo = GW ("druid_device_port_combo");
+	priv->device_speed = GW ("druid_device_speed_menu");
+	priv->device_speed_label = GW ("druid_device_speed_label");
+	priv->device_timeout = GW ("druid_device_timeout_spinner");
+	priv->device_usb = GW ("druid_usb_radio");
+	priv->device_irda = GW ("druid_irda_radio");
+	priv->device_network = GW ("druid_network_radio");
+
+#ifdef PILOT_LINK_0_12
+	/* usb: (libusb) pseudo-device is available from pilot-link 0.12.0 */
+	priv->libusb_list = NULL;
+	priv->libusb_label = gtk_list_item_new_with_label ("usb:");
+	gtk_widget_show(priv->libusb_label);
+	priv->libusb_list = g_list_append(priv->libusb_list, priv->libusb_label);
+	gtk_list_insert_items (GTK_LIST((GTK_COMBO(priv->device_port_combo))->list),
+	priv->libusb_list, 1);
+#endif
 
 	priv->pilot_info = GW ("pilot_user_frame");
 	priv->pilot_info_no = GW ("no_radio_button");
-	priv->pilot_username = GW ("pilot_username_entry");
-	priv->pilot_id = GW ("pilot_id_entry");
+	priv->pilot_username = GW ("druid_pilot_username_entry");
+	priv->pilot_id = GW ("druid_pilot_id_entry");
 	
 	/* FIXME: is this a libglade bug or what? if sync_label 
 	   is constructed in .glade it is not properly redrawn the first time. */
@@ -243,8 +270,8 @@ get_widgets (GnomePilotDruid *gpd)
 	priv->sync_label =  gtk_label_new ("");
 	gtk_box_pack_start (GTK_BOX (priv->sync_label_vbox), priv->sync_label, TRUE, FALSE, GNOME_PAD_SMALL);
 
-	priv->pilot_name = GW ("pilot_name_entry");
-	priv->pilot_basedir = GW ("pilot_basedir_entry");
+	priv->pilot_name = GW ("druid_pilot_name_entry");
+	priv->pilot_basedir = GW ("druid_pilot_basedir_entry");
 	
 #undef GW
 
@@ -302,10 +329,10 @@ init_widgets (GnomePilotDruid *gpd)
 
 	/* Main signals */
 	gtk_signal_connect (GTK_OBJECT (priv->druid_window), "delete_event",
-			    GTK_SIGNAL_FUNC (gpd_delete_window), gpd);
+	    GTK_SIGNAL_FUNC (gpd_delete_window), gpd);
 
 	gtk_signal_connect (GTK_OBJECT (priv->druid), "cancel",
-			    GTK_SIGNAL_FUNC (gpd_canceled), gpd);
+	    GTK_SIGNAL_FUNC (gpd_canceled), gpd);
 	gtk_signal_connect (GTK_OBJECT (priv->druid), "help",
 			    GTK_SIGNAL_FUNC (gpd_help), gpd);
 
@@ -313,6 +340,8 @@ init_widgets (GnomePilotDruid *gpd)
 	/* Page signals */
 	g_signal_connect_after (G_OBJECT (priv->page_cradle), "prepare",
 				G_CALLBACK (gpd_cradle_page_prepare), gpd);
+	g_signal_connect_after (G_OBJECT (priv->page_cradle), "next",
+				G_CALLBACK (gpd_cradle_page_next), gpd);
 
 	g_signal_connect_after (G_OBJECT (priv->page_sync), "prepare",
 				G_CALLBACK (gpd_sync_page_prepare), gpd);
@@ -330,6 +359,8 @@ init_widgets (GnomePilotDruid *gpd)
 			    GTK_SIGNAL_FUNC (gpd_device_info_check), gpd);
 	gtk_signal_connect (GTK_OBJECT (priv->device_port),"insert-text",
 			    GTK_SIGNAL_FUNC (insert_device_callback), NULL);
+	gtk_signal_connect (GTK_OBJECT (priv->device_network), "toggled",
+			    GTK_SIGNAL_FUNC (network_device_toggled_callback), gpd);
 	gtk_signal_connect (GTK_OBJECT (priv->device_port),"changed",
 			    GTK_SIGNAL_FUNC (gpd_device_info_check), gpd);
 
@@ -359,6 +390,9 @@ fill_widgets (GnomePilotDruid *gpd)
 	str = next_cradle_name (priv->state);
 	gtk_entry_set_text (GTK_ENTRY (priv->device_name), str);
 	g_free (str);
+	set_widget_visibility_by_type(gpd,
+	    GTK_TOGGLE_BUTTON(priv->device_network)->active ?
+	    PILOT_DEVICE_NETWORK : PILOT_DEVICE_SERIAL);
 	
 	fill_speed_menu (GTK_OPTION_MENU (priv->device_speed), 0);
 
@@ -406,6 +440,9 @@ cancel_dialog (GnomePilotDruid *gpd)
 	
 	priv = gpd->priv;
 
+	if (!priv->started)
+		return TRUE;
+
 	dlg = gtk_message_dialog_new (GTK_WINDOW (priv->druid_window), GTK_DIALOG_DESTROY_WITH_PARENT, 
 				      GTK_MESSAGE_QUESTION, GTK_BUTTONS_YES_NO,
 				      _("Setup did not complete and settings will not\n"
@@ -476,11 +513,36 @@ gpd_cradle_page_prepare (GnomeDruidPage *page, gpointer arg1, gpointer user_data
 {
 	GnomePilotDruid *gpd = GNOME_PILOT_DRUID (user_data);
 	gboolean ready;
-	
+
 	ready = check_cradle_settings (gpd);
 	gnome_druid_set_buttons_sensitive (GNOME_DRUID (arg1), TRUE, ready, TRUE, TRUE);
 }
 
+static gboolean
+gpd_cradle_page_next (GnomeDruidPage *page, gpointer arg1, gpointer user_data)
+{
+	GnomePilotDruid *gpd = GNOME_PILOT_DRUID (user_data);
+	GnomePilotDruidPrivate *priv;
+	GPilotDevice *tmp_device;
+	gboolean result;
+
+	priv = gpd->priv;
+
+	/* check the device settings */
+	tmp_device = gpilot_device_new();
+	read_device_config(GTK_OBJECT(gpd), tmp_device);
+	result = check_device_settings(tmp_device);
+	g_free(tmp_device->name);
+	g_free(tmp_device->port);
+	if (!result)
+		/* cancel proceeding to next page */
+		return TRUE;
+
+	priv->started = TRUE;
+
+	return FALSE;
+}
+
 static void
 gpd_sync_page_prepare (GnomeDruidPage *page, gpointer arg1, gpointer user_data)
 {
@@ -488,7 +550,7 @@ gpd_sync_page_prepare (GnomeDruidPage *page, gpointer arg1, gpointer user_data)
 	GnomePilotDruid *gpd = GNOME_PILOT_DRUID (user_data);
 	GnomePilotDruidPrivate *priv;
 	GNOME_Pilot_UserInfo user;
-	gchar *text;
+	gchar *text, *location;
 
 	priv = gpd->priv;
 
@@ -499,14 +561,15 @@ gpd_sync_page_prepare (GnomeDruidPage *page, gpointer arg1, gpointer user_data)
 	if (GTK_TOGGLE_BUTTON (priv->pilot_info_no)->active) {
 		/* do send_to_pilot */
 		read_pilot_config (GTK_OBJECT (gpd), priv->pilot);
-
-		text = g_strdup_printf (_("About to send the following data to the pilot.\n"
-				       "UserName: %s\nID: %d\n"
-				       "Please put pilot in %s (%s) and press HotSync button."),
+		location = priv->device->type == PILOT_DEVICE_NETWORK ?
+		    "netsync" : priv->device->port;
+		text = g_strdup_printf (_("About to send the following data to the PDA.\n"
+				       "Owner Name: %s\nPDA ID: %d\n"
+				       "Please put PDA in %s (%s) and press HotSync button."),
 					priv->pilot->pilot_username,
 					priv->pilot->pilot_id,
 					priv->device->name,
-					priv->device->port);
+					location);
 
 		save_config_and_restart (priv->gpc, priv->state);
 
@@ -522,10 +585,14 @@ gpd_sync_page_prepare (GnomeDruidPage *page, gpointer arg1, gpointer user_data)
 						  &priv->handle);
 	} else {
 		/* do get_from_pilot */
-		text = g_strdup_printf (_("About to retrieve UserName and ID from the pilot.\n"
-				       "Please put pilot in %s (%s) and press HotSync button."),
-					priv->device->name,
-					priv->device->port);
+		location = priv->device->type == PILOT_DEVICE_NETWORK ?
+		    "netsync" : priv->device->port;
+		text = g_strdup_printf (_("About to retrieve Owner Name and "
+					    "ID from the PDA.\n"
+					    "Please put PDA in %s (%s) and press "
+					    "HotSync button."),
+		    priv->device->name,
+		    location);
 
 		save_config_and_restart (priv->gpc, priv->state);
 
@@ -666,8 +733,8 @@ gpd_userinfo_requested (GnomePilotClient *gpc, const gchar *device, const GNOME_
 		g_free (priv->pilot->pilot_username);
 	priv->pilot->pilot_username = g_strdup (user->username);
 
-	text = g_strdup_printf (_("Successfully retrieved UserName and ID from pilot.\n"
-				  "UserName: %s\nID: %d"),
+	text = g_strdup_printf (_("Successfully retrieved Owner Name and ID from PDA.\n"
+				  "Owner Name: %s\nPDA ID: %d"),
 				priv->pilot->pilot_username,
 				priv->pilot->pilot_id);
 	gtk_label_set_text (GTK_LABEL (priv->sync_label), text);
@@ -698,3 +765,37 @@ gpd_destroy (GtkObject *object)
 	if (GTK_OBJECT_CLASS (parent_class)->destroy)
 		(* GTK_OBJECT_CLASS (parent_class)->destroy) (object);
 }
+
+static void
+network_device_toggled_callback (GtkRadioButton *network_radio, void *data)
+{
+	GnomePilotDruid *gpd = (GnomePilotDruid *)data;
+	GnomePilotDruidPrivate *priv;
+	int type;
+
+	priv = gpd->priv;
+
+	if(GTK_TOGGLE_BUTTON(network_radio)->active) {
+		type = PILOT_DEVICE_NETWORK;
+	} else {
+		type = PILOT_DEVICE_SERIAL;
+	}
+	
+	set_widget_visibility_by_type(gpd, type);
+}	
+
+static void
+set_widget_visibility_by_type(GnomePilotDruid *gpd, int type) {
+	GnomePilotDruidPrivate *priv;
+
+	priv = gpd->priv;
+
+	gtk_widget_set_sensitive(priv->device_port_combo,
+	    type != PILOT_DEVICE_NETWORK);
+	gtk_widget_set_sensitive(priv->device_port_label,
+	    type != PILOT_DEVICE_NETWORK);
+	gtk_widget_set_sensitive(priv->device_speed,
+	    type != PILOT_DEVICE_NETWORK);
+	gtk_widget_set_sensitive(priv->device_speed_label,
+	    type != PILOT_DEVICE_NETWORK);
+}
