@@ -27,6 +27,7 @@
 #include <unistd.h>
 #include <stdlib.h>
 #include <stdio.h>
+#include <string.h>
 #include "gnome-pilot-structures.h"
 #include "gpilot-gui.h"
 #include <libgnome/libgnome.h>
@@ -38,12 +39,11 @@
 #include <signal.h>
 
 /* From pi-csd */
-#include <signal.h>
 #include <sys/socket.h>
 #include <netinet/in.h>
-#include <net/if.h>
 #include <arpa/inet.h>
 #include <netdb.h>
+#include <pi-source.h>
 
 #define LOCK_DIR "/var/lock"
 #define LOCK_BINARY 0
@@ -300,7 +300,7 @@ gpilot_serial_device_init (GPilotDevice *device)
 		g_free (device->port);
 		device->port = NULL;
 
-		close (device->fd);
+		pi_close (device->fd);
 		device->fd = 0;
 
 		device->io = NULL;
@@ -318,7 +318,7 @@ static void
 gpilot_serial_device_free (GPilotDevice *device)
 {
 	if (device->fd) {
-		close (device->fd);
+		pi_close (device->fd);
 	}
 	g_free (device->name);
 	g_free (device->port);
@@ -328,10 +328,12 @@ gpilot_serial_device_free (GPilotDevice *device)
 static void
 gpilot_serial_device_deinit (GPilotDevice *device)
 {
+	GError *myerr = NULL;
+
 	if (device->io) {
 		g_source_remove (device->in_handle);
 		g_source_remove (device->err_handle);
-		g_io_channel_close (device->io);
+		g_io_channel_shutdown (device->io, 0, &myerr);
 		g_io_channel_unref (device->io);
 	}
 }
@@ -380,154 +382,6 @@ gpilot_irda_device_deinit (GPilotDevice *device)
 
 #ifdef WITH_NETWORK
 
-/* 
- * pi-csd.c: Connection Service Daemon, required for accepting 
- *           logons via NetSync (tm)
- * Copyright (c) 1997, Kenneth Albanowski
- *
- * While this function is useful in pi-csd, it is intended also to
- * be a demonstration of the proper (or improper, if I'm unlucky) techniques
- * to retrieve networking information.
- */
-static void
-fetch_host (char *hostname, int hostlen, struct in_addr *address,
-	   struct in_addr *mask)
-{
-#if defined (SIOCGIFCONF) && defined (SIOCGIFFLAGS)
-	int s, n, i;
-	struct ifconf ifc;
-	struct ifreq *ifr, ifreqaddr, ifreqmask;
-#endif
-	struct hostent *hent;
-
-#ifdef HAVE_GETHOSTNAME
-	/* Get host name the easy way */
-
-	gethostname (hostname, hostlen);
-#else
-# ifdef HAVE_UNAME
-	struct utsname uts;
-
-	if (uname (&uts) == 0) {
-		strncpy (hostname, uts.nodename, hostlen - 1);
-		hostname[hostlen - 1] = '\0';
-	}
-# endif	/*def HAVE_UNAME */
-#endif /*def HAVE_GETHOSTNAME */
-
-	/* Get host address through DNS */
-	hent = gethostbyname (hostname);
-
-	if (hent) {
-		while (*hent->h_addr_list) {
-			struct in_addr haddr;
-
-			memcpy (&haddr, *(hent->h_addr_list++), sizeof (haddr));
-			if (haddr.s_addr != inet_addr ("127.0.0.1"))
-				memcpy (address, &haddr, sizeof (haddr));
-		}
-	}
-#if defined (SIOCGIFCONF) && defined (SIOCGIFFLAGS)
-	s = socket (AF_INET, SOCK_DGRAM, 0);
-
-	if (s < 0)
-		return;
-
-	ifc.ifc_buf = calloc (1024, 1);
-	ifc.ifc_len = 1024;
-
-	if (ioctl (s, SIOCGIFCONF, (char *) &ifc) < 0)
-		goto done;
-
-	n = ifc.ifc_len;
-	for (i = 0; i < n; i += ifreq_size (*ifr)) {
-		struct sockaddr_in *a;
-		struct sockaddr_in *b;
-
-		ifr = (struct ifreq *) ((caddr_t) ifc.ifc_buf + i);
-		a = (struct sockaddr_in *) &ifr->ifr_addr;
-		strncpy (ifreqaddr.ifr_name, ifr->ifr_name, sizeof (ifreqaddr.ifr_name));
-		strncpy (ifreqmask.ifr_name, ifr->ifr_name, sizeof (ifreqmask.ifr_name));
-
-		if (ioctl (s, SIOCGIFFLAGS, (char *) &ifreqaddr) < 0)
-			continue;
-
-		/* Reject loopback device */
-#ifdef IFF_LOOPBACK
-		if (ifreqaddr.ifr_flags & IFF_LOOPBACK)
-			continue;
-#endif /*def IFF_LOOPBACK */
-
-#ifdef IFF_UP
-		/* Reject down devices */
-		if (!(ifreqaddr.ifr_flags & IFF_UP))
-			continue;
-#endif /*def IFF_UP */
-
-		if (ifr->ifr_addr.sa_family != AF_INET)
-			continue;
-
-		/* If it is a point-to-point device, use the dest address */
-#if defined (IFF_POINTOPOINT) && defined (SIOCGIFDSTADDR)
-		if (ifreqaddr.ifr_flags & IFF_POINTOPOINT) {
-			if (ioctl (s, SIOCGIFDSTADDR, (char *) &ifreqaddr) < 0)
-				break;
-
-			a = (struct sockaddr_in *) &ifreqaddr.ifr_dstaddr;
-
-			if (address->s_addr == 0) {
-				memcpy (address, &a->sin_addr, sizeof (struct in_addr));
-			}
-		}
-		else
-#endif /*defined (IFF_POINTOPOINT) && defined (SIOCGIFDSTADDR) */
-			/* If it isn't a point-to-point device, use the address */
-#ifdef SIOCGIFADDR
-		{
-			if (ioctl (s, SIOCGIFADDR, (char *) &ifreqaddr) < 0)
-				break;
-
-			a = (struct sockaddr_in *) &ifreqaddr.ifr_addr;
-
-			if (address->s_addr == 0) {
-				memcpy (address, &a->sin_addr, sizeof (struct in_addr));
-			}
-		}
-#endif /*def SIOCGIFADDR */
-		/* OK, we've got an address */
-
-		/* Compare netmask against the current address and see if it seems to match. */
-#ifdef SIOCGIFNETMASK
-		if (ioctl (s, SIOCGIFNETMASK, (char *) &ifreqmask) < 0)
-			break;
-
-/* Is there any system where we need to use ifr_netmask?  */
-#if 1
-		b = (struct sockaddr_in *) &ifreqmask.ifr_addr;
-#else
-		b = (struct sockaddr_in *) &ifreqmask.ifr_netmask;
-#endif
-
-		if ((mask->s_addr == 0) && (address->s_addr != 0)) {
-			if ((b->sin_addr.s_addr & a->sin_addr.s_addr) ==
-			    (b->sin_addr.s_addr & address->s_addr)) {
-				memcpy (mask, &b->sin_addr, sizeof (struct in_addr));
-
-				/* OK, we've got a netmask */
-
-				break;
-			}
-		}
-#endif /*def SIOCGIFNETMASK */
-
-	}
-
- done:
-	free (ifc.ifc_buf);
-	close (s);
-#endif /*defined (SIOCGIFCONF) && defined (SIOCGIFFLAGS) */
-}
-
 /* gpilot_network_device_init (GPilotDevice *device)
  * pi-csd.c: Connection Service Daemon, required for accepting 
  *           logons via NetSync (tm)
@@ -537,85 +391,71 @@ fetch_host (char *hostname, int hostlen, struct in_addr *address,
 static gint 
 gpilot_network_device_init (GPilotDevice *device)
 {
-/*
- * Open a file descriptor. device->fd
- * Check fd. return -1 if fail.
- * device->io = g_io_channel_unix_new (device->fd);
- * g_io_channel_ref (device->io);
- * return 0;
- *
- */
+#ifndef PILOT_LINK_0_12
+	struct pi_sockaddr addr;
+#endif
 
-/* char hostname[130];	this is device->host */
-	struct in_addr address, netmask;
+	char pi_net[100];
+	int ret, fd;
 
-
-/*   int sockfd;	this is device->fd */
-	struct sockaddr_in serv_addr;
-
-	struct hostent *hent;
-/*   int quiet = 0;		It's Always quiet here */
-
-	memset (&address, 0, sizeof (address));
-	memset (&netmask, 0, sizeof (netmask));
-
-/*   hostname[0] = 0; */
-/* device->host = g_new (gchar, 130);
-   g_free (device->host);
-*/
-
-/* This is used if the hostname is not correct in the conf file
-   I'll move it to the conf file saving later */
-	fetch_host (device->host, 128, &address, &netmask); 
-	
-/* device->host I already have from the config file */
-
-	if (inet_aton (device->ip, &address) == 0) /* Invalid */
-	{
-		if ((hent = gethostbyname (device->ip))) {
-			memcpy (&address.s_addr, hent->h_addr, sizeof (address));
-		} else {
-			g_message ("Invalid ip address '%s'", device->ip);
-			return -1;
+	memset(pi_net, 0, sizeof(pi_net));
+	strncpy(pi_net, "net:", 4);
+	if (device->ip != NULL && (*device->ip != 0x0)) {
+		struct 	sockaddr_in serv_addr;
+		/* Verify the IP address is valid */
+		memset(&serv_addr, 0, sizeof(serv_addr));
+		serv_addr.sin_family = AF_INET;
+		serv_addr.sin_addr.s_addr = inet_addr(device->ip);
+		if (serv_addr.sin_addr.s_addr == (in_addr_t)-1) {
+			struct hostent *hostent = gethostbyname(device->ip);
+		
+			if (!hostent) {
+				g_warning ("Device [%s]: Bad IP address/hostname: %s", 
+				    device->name, device->ip);
+				return -1;
+			}
 		}
+		
+		strncat(pi_net, device->ip, sizeof(pi_net) - 2 - strlen(pi_net));
+	} else {
+		strncat(pi_net, "any", 3);
 	}
-
-	if (inet_aton (device->netmask, &netmask) == 0) {
-		g_message ("Invalid netmask '%s'", device->netmask);
-		return -1;
-	}
-
-	/* cannot execute without address and hostname */
-	if ((address.s_addr == 0) || (strlen (device->host) == 0))
-	{
-		g_message ("Cannot execute without ip address and hostname.");
-		return -1;
-	}
-
-	device->fd = socket (AF_INET, SOCK_DGRAM, 0);
+#ifdef PILOT_LINK_0_12
+	device->fd = pi_socket (PI_AF_PILOT, PI_SOCK_STREAM, PI_PF_DLP);
+#else
+	device->fd = pi_socket (PI_AF_PILOT, PI_SOCK_STREAM, PI_PF_NET);
+#endif
 	if (device->fd < 0) {
-		g_message ("Unable to get socket");
+		g_warning ("Device [%s, %s]: Unable to get socket: %s", 
+		    device->name, pi_net, strerror(errno));
 		return -1;
 	}
 
-	memset (&serv_addr, 0, sizeof (serv_addr));
-	serv_addr.sin_family = AF_INET;
-	serv_addr.sin_addr.s_addr = htonl (INADDR_ANY);
-	serv_addr.sin_port = htons (14237);
+#ifdef PILOT_LINK_0_12
+	ret = pi_bind (device->fd, pi_net);
+#else
+	strncpy (addr.pi_device,pi_net, strlen(pi_net));
 
-	if (bind (device->fd, (struct sockaddr *) &serv_addr, sizeof (serv_addr)) < 0) {
-		g_message ("Unable to bind socket");
+	ret = pi_bind (device->fd, (struct sockaddr*)&addr, sizeof (addr));
+#endif
+	if (ret < 0) {
+		g_warning ("Device [%s, %s]: Unable to bind socket: err %d",
+		    device->name, pi_net, ret);
 		return -1;
 	}
-	/*
-	g_message ("Connection Service Daemon for Palm Computing (tm) device active.");
-	g_message ("Accepting connection requests for '%s' at %s with mask %s", 
-		   device->host, inet_ntoa (address), inet_ntoa (netmask));
-	*/
 
-	/* We don't want to infinite loops here */
-	/* Lets try something else instead */
-	device->io = g_io_channel_unix_new (device->fd);
+	/* Now listen for incoming connections */
+	if (pi_listen (device->fd, 1) != 0) {
+		g_warning ("Device [%s, %s]: Error from listen: %s",
+		    device->name, pi_net, strerror (errno));
+		pi_close(device->fd);
+		return -1;
+	}
+	/* Register an interest in the socket, to get events when devices connect */
+	fd = dup(device->fd);
+	fcntl(fd, F_SETFD, FD_CLOEXEC); /* don't let bonobo keep references alive */
+	fcntl(device->fd, F_SETFD, FD_CLOEXEC);
+	device->io = g_io_channel_unix_new (fd);
 	g_io_channel_ref (device->io);
 
 	/* All went well.. */
@@ -626,17 +466,29 @@ static void
 gpilot_network_device_free (GPilotDevice *device)
 {
 	if (device->fd) {
-		close (device->fd);
+		pi_close (device->fd);
 	}
 
 	g_free(device->ip);
-	g_free(device->host);
-	g_free(device->netmask);
 }
 
 static void
 gpilot_network_device_deinit (GPilotDevice *device)
 {
+	GError *err = NULL;
+
+	if (device->io) {
+		g_source_remove (device->in_handle);
+		g_source_remove (device->err_handle);
+		g_io_channel_shutdown (device->io, 0, &err);
+		if (err != NULL)
+		{
+			/* Report error to user, and free error */
+			g_warning ("error from shutdown: %s\n", err->message);
+			g_error_free (err);
+		} 
+		g_io_channel_unref (device->io);
+	}
 }
 #endif /* WITH_NETWORK */
 
@@ -680,8 +532,6 @@ static void
 gpilot_network_device_load (GPilotDevice *device)
 {
 	device->ip = gnome_config_get_string ("ip");
-	device->host = gnome_config_get_string ("host");
-	device->netmask = gnome_config_get_string ("netmask");
 }
 
 gint
@@ -784,7 +634,7 @@ get_devices (void)
 	if (n==0) {
 		g_warning (_("Number of devices is configured to 0"));
 #ifdef GUI
-		gpilot_gui_warning_dialog (_("No cradles configure.\n"
+		gpilot_gui_warning_dialog (_("No devices configured.\n"
 					    "Please run gpilotd-control-applet (use gnomecc)\n" 
 					    "to configure gnome-pilot."));    
 #endif
@@ -820,9 +670,9 @@ get_pilots (void)
 	n = gnome_config_get_int ("/gnome-pilot.d/gpilotd/General/num_pilots=0");
 
 	if (n==0) {
-		g_warning (_("Number of pilots is configured to 0"));
+		g_warning (_("Number of PDAs is configured to 0"));
 #ifdef GUI
-		gpilot_gui_warning_dialog (_("No pilots configured.\n"
+		gpilot_gui_warning_dialog (_("No PDAs configured.\n"
 					    "Please run gpilotd-control-applet (use gnomecc)\n" 
 					    "to configure gnome-pilot."));    
 #endif
