@@ -1084,7 +1084,12 @@ hal_device_added (LibHalContext *ctx, const char *udi)
 		if (device->type == PILOT_DEVICE_USB_VISOR) {
 			if (!visor_net)
 				device->type = PILOT_DEVICE_SERIAL;
-			
+			/* problems have been reported with devices
+ 			 * not being ready for sync immediately,
+ 			 * so we wait for 0.4 seconds.  See
+ 			 * bugzilla.gnome #362565
+ 			 */
+			usleep(400000);
 			/* just try to sync.  Until I can talk to 
 			 * the kernel guys this is the best way to 
 			 * go. */
@@ -1313,6 +1318,7 @@ visor_devices_timeout (gpointer data)
 				}
 				vend_id = g_strndup (fcontent, 4);
 				g_free (fname);
+				g_free (fcontent);
 
 				fname = g_build_filename (sysfs_dir_name, entry,
 				    f_prod, NULL);
@@ -1327,6 +1333,7 @@ visor_devices_timeout (gpointer data)
 				}
 				prod_id = g_strndup (fcontent, 4);
 				g_free (fname);
+				g_free (fcontent);
 
 				to_match = g_strconcat ("Vendor=", vend_id,
 				    " ProdID=", prod_id, NULL);
@@ -1438,7 +1445,7 @@ monitor_channel (GPilotDevice *dev, GPilotContext *context)
 	}
 
 	if (dev->type == PILOT_DEVICE_NETWORK) {
-		g_message (_("Watching %s (%s)"), dev->name, dev->ip);
+		g_message (_("Watching %s (network)"), dev->name);
 	} else {
 		g_message (_("Watching %s (%s)"), dev->name, dev->port);
 	}
